package com.itko.lisa.ext.gxs.filter;

import java.util.Collection;
import java.util.LinkedList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.itko.lisa.test.FilterBaseImpl;
import com.itko.lisa.test.TestDefException;
import com.itko.lisa.test.TestExec;
import com.itko.lisa.test.TestRunException;
import com.itko.util.Parameter;
import com.itko.util.ParameterList;
import com.itko.util.XMLUtils;
import com.itko.lisa.vse.stateful.model.*;

public class TrustedSendSessionFilter extends FilterBaseImpl {

	private static final long serialVersionUID = 4702100012911701503L;

	private static final transient Log log = LogFactory
			.getLog(TrustedSendSessionFilter.class);

	public static final String FILTER_TITLE = "GXS Trusted Send Session Filter";
	private static final String ILOG_PARAM = "iLog";
	private static final String ILOG_LABEL = "iLog Value:";
	private String iLogValue;
	private static final String PROP_PARAM = "prop";
	private static final String PROP_LABEL = "Property Name:";
	private String prop;

	/**
	 * Get the display name for this filter.
	 * 
	 * @return a <code>String</code>
	 */
	public String getTypeName() {
		return FILTER_TITLE;
	}

	/**
	 * Build the list of parameters for this filter.
	 * 
	 * @return a <code>ParameterList</code>
	 */
	public ParameterList getParameters() {
		ParameterList pl = new ParameterList();

		pl.addParameter(new Parameter(ILOG_LABEL,
									 ILOG_PARAM, 
									 iLogValue,
									 String.class));
		
		pl.addParameter(new Parameter(PROP_LABEL,
									 PROP_PARAM, 
									 prop,
									 String.class));

		return pl;
	}
	
	/**
     * Once constructed, we take the XML Element and get our field data from
     * there.
     * 
     * @param e
     * @throws TestDefException
     */
    public void initialize(org.w3c.dom.Element e) throws TestDefException
    {
        iLogValue = XMLUtils.getAttributeOrChildText(e, ILOG_PARAM);
        prop = XMLUtils.getAttributeOrChildText(e, PROP_PARAM);
    }
    
	@SuppressWarnings("unchecked")
	public Collection getFilterNodeConnections(ParameterList parameterList) {
		return null;
	}
	
    /**
     * @param testExec
     * @return a <code>boolean</code>
     * @throws TestRunException
     */
	@Override
    public boolean subPreFilter(TestExec testExec) throws TestRunException
    {
        // nothing to do here
        return false;
    }
	
	@Override
	public boolean subPostFilter(TestExec ts) throws TestRunException {
		
		Response r = new Response();
		String body = "<lisa.vse.java.response>\n" +
			"<lisa.vse.java.response.result>\n" +
			"<long>" +
			ts.parseInState(iLogValue) +
			"</long>\n" +
			"</lisa.vse.java.response.result>\n" +
			"</lisa.vse.java.response>\n";
		r.setBody(body);
		TransientResponse tr = new TransientResponse(r);
		LinkedList<TransientResponse> l = new LinkedList<TransientResponse>();
		l.add(tr);
		ts.setStateValue(prop, l);
		return false;
	}
}
