package com.itko.lisa.ext.gxs.step;

import com.google.common.base.Preconditions;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.io.Files;
import com.itko.lisa.remote.IAgentInfo;
import com.itko.lisa.remote.client.AgentClient;
import com.itko.lisa.remote.utils.UUID;
import com.itko.lisa.test.CustJavaNodeInterface;
import com.itko.lisa.test.TestCase;
import com.itko.lisa.test.TestDefException;
import com.itko.lisa.test.TestExec;
import com.itko.lisa.test.TestRunException;
import com.itko.util.Parameter;
import com.itko.util.ParameterList;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;

import java.io.File;
import java.util.Map;

/**
 * CopyFileToAgent - Simple step to copy a file into the agent directory of a remote agent
 *
 * @author <a href="mailto:daniel.bingham@itko.com">Daniel Bingham</a>
 */
@SuppressWarnings( {"MissingFieldJavaDoc"} )
public class CopyFileToAgent implements CustJavaNodeInterface
{
    private static final Log log = LogFactory.getLog( CopyFileToAgent.class );

    protected static final String AGENT_NAME       = "Agent Names";
    protected static final String AGENT_NAME_KEY   = "Agent_Names";
    protected static final String agentNameDefault = "";

    protected static final String FILE_TO_COPY      = "File To Copy";
    protected static final String FILE_TO_COPY_KEY  = "File_To_Copy";
    protected static final String fileToCopyDefault = "";

    protected static final String UNIQUE_KEY       = "Unique Key (Optional)";
    protected static final String UNIQUE_KEY_KEY   = "Unique_Key";
    protected static final String uniqueKeyDefault = UUID.newUUID();

    /**
     * initialize
     *
     * @param testCase the testCase document
     * @param element  the element
     *
     * @throws TestDefException
     */
    @Override
    public void initialize( final TestCase testCase, final Element element ) throws TestDefException
    {}

    /**
     * getParameters
     *
     * @return a ParameterList
     */
    @Override
    public ParameterList getParameters()
    {
        ParameterList params = new ParameterList();

        params.addParameter( new Parameter( AGENT_NAME,
                                            AGENT_NAME_KEY,
                                            agentNameDefault,
                                            String.class ) );

        params.addParameter( new Parameter( FILE_TO_COPY,
                                            FILE_TO_COPY_KEY,
                                            fileToCopyDefault,
                                            String.class ) );
        params.addParameter( new Parameter( UNIQUE_KEY,
                                            UNIQUE_KEY_KEY,
                                            uniqueKeyDefault,
                                            String.class ) );

        return params;
    }

    /**
     * executeNodeLogic
     *
     * @param testExec the testExec
     * @param map      the Map of parameters for the step
     *
     * @return the result of the step
     * @throws TestRunException
     */
    @SuppressWarnings( {"RawUseOfParameterizedType"} )
    @Override
    public Object executeNodeLogic( final TestExec testExec, final Map map ) throws TestRunException
    {
        String agentList = (String) map.get( AGENT_NAME_KEY );
        Iterable<String> agents = Splitter.on( "," ).trimResults().omitEmptyStrings().split( agentList );

        String fileToCopy = (String) map.get( FILE_TO_COPY_KEY );
        File sourceFile = new File( fileToCopy );

        String uniqueKey = "agentTxfr-" + Strings.nullToEmpty( (String) map.get( UNIQUE_KEY_KEY ) );

        Preconditions.checkArgument( sourceFile.exists(),
                                     "%s must be a file which exists (Note: the path must be provided and correct)",
                                     fileToCopy );

        String message = "Couldn't determine the where the created file is (if one was created at all)";

        for( String agent : agents )
        {
            IAgentInfo handle = AgentClient.getInstance().getRemoteAgentInfo( agent );

            File tempDir = Files.createTempDir();
            File temp;
            String agentDir;
            try
            {
                System.out.println( "Temp dir:" + tempDir.getCanonicalPath() );
                temp = new File( tempDir.getCanonicalPath() + "/" + uniqueKey + ".tmp" );
                Files.copy( new File( fileToCopy ), temp );

                AgentClient.getInstance().uploadExtension( handle, temp.getCanonicalPath() );

                agentDir = (String) AgentClient.getInstance().eval( handle, "return _agent.getAgentDir()" );
            }
            catch( Exception e )
            {
                log.error( e.getMessage(), e );
                throw new TestRunException( e.getMessage(), e );
            }

            // because this isn't actually a constant condition...
            //noinspection ConstantConditions
            if( temp != null &&
                agentDir != null )
            {
                message = "File was created on the remote agent at " + agentDir + "/" + temp.getName();
            }
        }

        return message;
    }
}
