package com.itko.lisa.ext.gxs.step;

import java.io.Serializable;
import java.util.Map;

import org.w3c.dom.Element;

import com.itko.lisa.test.CustJavaNodeInterface;
import com.itko.lisa.test.TestCase;
import com.itko.lisa.test.TestDefException;
import com.itko.lisa.test.TestExec;
import com.itko.lisa.test.TestRunException;
import com.itko.util.ParameterList;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.itko.lisa.remote.*;
import com.itko.lisa.remote.client.*;
import com.itko.lisa.remote.plumbing.*;

public class TgesSendToWm implements CustJavaNodeInterface {

	private static final long serialVersionUID = -8599070375950356462L;
	private static Log log = LogFactory.getLog(TgesSendToWm.class);
    
	/**
     * Field PARAM_DELIMITER
     */
    private static final String PARAM_DELIMITER = "=&";
    protected static final String TGES_AGENT_NAME = "TGES Agent Name";
    protected static final String WM_RPC_LISTENER = "webMethods RPC Listener";
    protected static final String WM_USER_ID = "webMethods User ID";
    protected static final String WM_PASSWORD = "webMethods Password";
    protected static final String WM_FOLDER = "webMethods Folder";
    protected static final String WM_SERVICE = "webMethods Service";
    protected static final String EVENT_XML = "Event XML";
    protected static final String ILOG = "iLog";
    
    protected static final String SCRIPT = "import com.gxs.iefe.tge.TgesToWm;" +
		"TgesToWm callHandler = new TgesToWm($1, $2, $3, $4, $5);" +
		"callHandler.dumpDirectToWm($6, false, $7);" +
		"return \"Executed dumpDirectToWm for iLog $7\";";
    
    protected static final long timeout = 30000;
    
	@Override
	public Object executeNodeLogic(TestExec ts, Map map)
			throws TestRunException {
		
		String wmRPCListener = ts.parseInState((String)map.get(WM_RPC_LISTENER));
		String wmUserId = ts.parseInState((String)map.get(WM_USER_ID));
		String wmPassword = ts.parseInState((String)map.get(WM_PASSWORD));
		String wmFolder = ts.parseInState((String)map.get(WM_FOLDER));
		String wmService = ts.parseInState((String)map.get(WM_SERVICE));
		String eventXml = ts.parseInState((String)map.get(EVENT_XML));
		String iLog = ts.parseInState((String)map.get(ILOG));
		
		String tgesAgentName = ts.parseInState((String)map.get(TGES_AGENT_NAME));
		
		// Get the handle for our Agent
		IAgentInfo handle = AgentClient.getInstance().getRemoteAgentInfo(tgesAgentName);
		Serializable[] inputs = new Serializable[] {
				wmRPCListener,
				wmUserId,
				wmPassword,
				wmFolder,
				wmService,
				eventXml,
				iLog
		};
		
		JMSMethodInvocation jmi = new JMSMethodInvocation(SCRIPT, inputs, timeout);
		try {
			return JMSConsoleClient.invoke(handle, jmi);
		} catch (JMSInvocationException e) {
			return "Unable to invoke TGES Agent: " + e.getMessage();
		} 
	}

	@Override
	public ParameterList getParameters() {
        StringBuffer paramBuffer = new StringBuffer();

        paramBuffer.append( TGES_AGENT_NAME )
                .append( PARAM_DELIMITER )
                .append( WM_RPC_LISTENER )
                .append( PARAM_DELIMITER )
                .append( WM_USER_ID )
                .append( PARAM_DELIMITER )
                .append( WM_PASSWORD )
                .append( PARAM_DELIMITER )
                .append( WM_FOLDER )
                .append( PARAM_DELIMITER )
                .append( WM_SERVICE )
                .append( PARAM_DELIMITER )
                .append( EVENT_XML )
                .append( PARAM_DELIMITER )
                .append( ILOG )
                .append( PARAM_DELIMITER );

        return new ParameterList( paramBuffer.toString() );
	}

	@Override
	public void initialize(TestCase arg0, Element arg1) throws TestDefException {
		log.debug( "initialize: start" );		
	}

}
