package com.itko.lisa.ext.gxs.vse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.itko.lisa.vse.stateful.common.DataProtocolDynamicEditor;
import com.itko.lisa.vse.stateful.model.MatchTolerance;
import com.itko.lisa.vse.stateful.model.Request;
import com.itko.lisa.vse.stateful.model.Response;
import com.itko.lisa.vse.stateful.model.TransientResponse;
import com.itko.lisa.vse.stateful.protocol.DataProtocol;
import com.itko.util.Parameter;
import com.itko.util.ParameterList;

/**
 * GXS DataProtocol
 * 
 * @author <a href="mailto:luther@itko.com">Luther Birdzell</a>
 */
public class GxsDataProtocolHandler
        extends DataProtocol
{
 	private static final transient Log log = LogFactory
			.getLog(GxsDataProtocolHandler.class);

    /**
     * This method is used to allow the data protocol to update the specified
     * request. At this time, the current implementation invokes the old style
     * method. Subclasses must overrided this method soon so that the older one
     * can be removed. When that happens, this method will become an empty
     * implementation.
     * 
     * @param request
     *            the request to update.
     */
    public void updateRequest(Request request)
    {
        String text = request.isBinary() ? null : request.getBody();
        byte[] bytes = request.getBodyBytes();

        log.info("Request text: '" + text + "'");
        log.info("Request bytes: '" + bytes + "'");
        
        // impl biz logic here
    }

    /**
     * This method is used to allow the data protocol to update the specified
     * response during recording. At this level, we don't do anything.
     * 
     * @param response
     *            the response to update.
     */
    public void updateResponse(Response response)
    {
    }

    /**
     * This method is used to allow the data protocol to update the specified
     * response during playback. At this level, we don't do anything.
     * 
     * @param response
     *            the response to update.
     */
    public void updateResponse(TransientResponse response)
    {
    }

    /**
     * This method should be used by subclasses to update the given default
     * response for unknown conversational requests. At this level, no
     * additional information is added.
     * 
     * @param response
     *            the response to use for unknown conversational requests.
     */
    public void updateUnknownConversationalResponse(Response response)
    {
    }

    /**
     * This method is used to allow the DataProtocols to provide dynamic
     * parameters they need at runtime.
     * 
     * @return XML that the standard LISA framework will write into a VSM.
     */
    public String getDynamicParametersXML()
    {
        return "";
    }

    /**
     * This enables a dataprotocol to be initialized with parameters at VSM
     * runtime to modify request objects. This gets called by the LISA
     * framework. First version will be from Filters, but at some point this may
     * get more embedded into the VSE framework.
     * 
     * This works similar to Filters in LISA so everything is done with
     * ParameterLists. In this first version the common filter
     * "DataProtocolDynamicParametersControllFilter" pulls the one parameters it
     * needs so you can assume the entire List is your parameters.
     * 
     * @param pl
     */
    public void initialize(ParameterList pl)
    {
    }

    /**
     * The partner to initialize which allows the dataprotocol to persist it's
     * info.
     */
    public ParameterList getParameters()
    {
        return null;
    }

    public void setParameters(ParameterList pl)
    {
        // nothing
    }

    /**
     * Provide a custom editor for the runtime side of things if you like.
     * Otherwise the standard ParameterList editor will render.
     * 
     * @param pl
     * @return
     */
    public DataProtocolDynamicEditor getCustomEditor(ParameterList pl)
    {
        return null;
    }

    /**
     * Easy way to determine if this dataprotocol would like to use dynamic
     * properties.
     * 
     * @return
     */
    public boolean isDynamicParameterCapable()
    {
        return false;
    }
}
