package com.itko.lisa.ext.trilead;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.pool.*;
import org.apache.commons.pool.impl.*;
import org.w3c.dom.Element;

import com.itko.lisa.test.CustJavaNodeInterface;
import com.itko.lisa.test.TestCase;
import com.itko.lisa.test.TestDefException;
import com.itko.lisa.test.TestEvent;
import com.itko.lisa.test.TestExec;
import com.itko.lisa.test.TestRunException;
import com.itko.util.ParameterList;
import java.io.IOException;

import com.trilead.ssh2.*;

@Deprecated
public class SFTPSimpleStep implements CustJavaNodeInterface {

	private static final long serialVersionUID = -7355005808610859234L;
	
    private static Log log = LogFactory.getLog(SFTPSimpleStep.class);
    
	public static final String PARAM_DELIMITER = "=&";
	public static final String HOSTNAME = "Hostname";
	public static final String USERNAME = "User";
	public static final String PASSWORD = "Password";
	public static final String COMMAND = "Command (ls, read, overwrite, forcewrite, create, delete)";
	public static final String PATH = "Path";
	public static final String FILENAME = "Filename (not for ls)";
	public static final String CONTENT = "File content (overwrite, forcewrite, create only)";

	public static final String CMD_LS = "ls";
	public static final String CMD_READ = "read";
	public static final String CMD_OVERWRITE = "overwrite";
	public static final String CMD_FORCEWRITE = "forcewrite";
	public static final String CMD_CREATE = "create";
	public static final String CMD_DELETE = "delete";

	private static Map<Integer,GenericObjectPool> poolMap = new HashMap<Integer,GenericObjectPool>();
	
	@SuppressWarnings("unchecked")
	public Object executeNodeLogic(TestExec testExec, Map params)
			throws TestRunException {
		
		// Try to get the connection settings
		SFTPStepSettings settings = new SFTPStepSettings();
		
		try {
			settings.setHostname(testExec.parseInState(BeanUtils.getProperty(params, HOSTNAME)));
			settings.setUsername(testExec.parseInState(BeanUtils.getProperty(params, USERNAME)));
			settings.setPassword(testExec.parseInState(BeanUtils.getProperty(params, PASSWORD)));
			settings.setCommand(testExec.parseInState(BeanUtils.getProperty(params, COMMAND)));
			settings.setPath(testExec.parseInState(BeanUtils.getProperty(params, PATH)));
			settings.setFilename(testExec.parseInState(BeanUtils.getProperty(params, FILENAME)));
			settings.setContent(testExec.parseInState(BeanUtils.getProperty(params, CONTENT)));
			
		} catch (Exception beanUtilsException) {
			// Throw the exception along
            log.error(beanUtilsException.getMessage(), beanUtilsException);
            throw new RuntimeException(beanUtilsException);
		}
		
		// Create the SFTP connection
		SFTPv3Client client = null;
		try {
			
			// Let's check to see if someone else has mad a SFTPv3Client
			// to our same host that we can borrow, so we call getClient()
			client = getClient(settings, testExec);
			
			// Some logging
			String msg = "SFTP: Attempting to "
				+ settings.getCommand()
				+ " path=(" + settings.getPath() + ")"
				+ " file=(" + settings.getFilename() + ")";
			log.debug(msg);
			testExec.raiseEvent(TestEvent.EVENT_NODEMSG, testExec.getCurrentNode().getName(), msg);
			
			// Figure out the correct behavior
			String command = settings.getCommand();
			if (command.equalsIgnoreCase(CMD_LS)) {
				msg = cmd_ls(client, settings);
			} else if (command.equalsIgnoreCase(CMD_READ)) {
				msg = cmd_read(client, settings);
			} else if (command.equalsIgnoreCase(CMD_OVERWRITE)) {
				msg = cmd_overwrite(client, settings);
			} else if (command.equalsIgnoreCase(CMD_FORCEWRITE)) {
				msg = cmd_forcewrite(client, settings);
			} else if (command.equalsIgnoreCase(CMD_CREATE)) {
				msg = cmd_create(client, settings);
			} else if (command.equalsIgnoreCase(CMD_DELETE)) {
				msg = cmd_delete(client, settings);
			} else {
				log.error("Unknown command: " + command);
				throw new RuntimeException("Unknown command: " + command);
			}
			
			// Cleanup and return
			//testExec.raiseEvent(TestEvent.EVENT_EXECTIME, "1234", null);
			testExec.raiseEvent(TestEvent.EVENT_EXECTIME, testExec.getCurrentNode().getName(), "9999", false);
			Thread.sleep(2000);
			testExec.raiseEvent(TestEvent.EVENT_NODERESPONSE, testExec.getCurrentNode().getName(), msg);
			return msg;
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new RuntimeException(e);
		} finally {
			cleanupClient(client, settings, testExec);
		}
	}
	
	/**
	 * 
	 * @param settings
	 * @return
	 * @throws IOException
	 */
	private SFTPv3Client getClient(SFTPStepSettings settings, TestExec testExec) throws Exception {
		
		log.debug("SFTP: getClient() START TestExec#" + testExec.getInstance());
		
		// Try to find the client pool
		GenericObjectPool clientPool = poolMap.get(settings.hashCode());
		if (clientPool == null) {
			
			log.debug("SFTP: createPool START TestExec#" + testExec.getInstance());
			
			// Create a new factory and pool, then add to the hashmaps
			String hostname = testExec.parseInState(settings.getHostname());
			String username = testExec.parseInState(settings.getUsername());
			String password = testExec.parseInState(settings.getPassword());
			TrileadConnectionFactory factory = new TrileadConnectionFactory(hostname, username, password, TrileadConnectionFactory.AUTH_PASSWORD, null, null);
			clientPool = new GenericObjectPool(factory, 9);
			poolMap.put(settings.hashCode(), clientPool);
			
			log.debug("SFTP: createPool FINISH TestExec#" + testExec.getInstance());
		}
		
		//String hostname = settings.getHostname();
		SFTPv3Client client = (SFTPv3Client)clientPool.borrowObject();
		log.debug("SFTP: getClient() FINISH TestExec#" + testExec.getInstance());
		return client;
	}
	
	/**
	 * 
	 * @param client
	 * @param settings
	 */
	private void cleanupClient(SFTPv3Client client, SFTPStepSettings settings, TestExec testExec) {
		log.debug("SFTP: cleanupClient() TestExec#" + testExec.getInstance());
		try {
			ObjectPool clientPool = poolMap.get(settings.hashCode());
			clientPool.returnObject(client);
		} catch (Exception e) {
			log.debug(e.getMessage(), e);
		}
	}

	/**
	 * 
	 * @param client
	 * @return
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	private String cmd_ls(SFTPv3Client client, SFTPStepSettings settings) throws IOException {
		String path = settings.getPath();
		
		// Create the result value
		StringBuffer sbFiles = new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sbFiles.append("<directory path=\"" + path + "\">");
		
		Vector files = client.ls(path);
		for(int i=0; i < files.size(); i++) {
			SFTPv3DirectoryEntry entry = (SFTPv3DirectoryEntry)files.get(i);
			
			// If it contains 0040000 it is a directory
			if ((entry.attributes.permissions.intValue() & 0040000) != 0) {
				sbFiles.append("<directory>" + entry.filename + "</directory>");
			} else {
				sbFiles.append("<file>" + entry.filename + "</file>");
			}
		}
		sbFiles.append("</directory>");
		return sbFiles.toString();
	}
	
	/**
	 * 
	 * @param client
	 * @return
	 * @throws IOException
	 */
	private String cmd_read(SFTPv3Client client, SFTPStepSettings settings) throws IOException {
		String path = settings.getPath();
		String filename = settings.getFilename();
		
		// Create the result value
		StringBuffer sbFile = new StringBuffer();
		byte[] dst = new byte[32768];
		int fileOffset = 0;
		int len = 0;
		
		// Create the handle
		SFTPv3FileHandle handle = client.openFileRO(path + "/" + filename);
		
		// Read the file
		while (len != -1) {
			len = client.read(handle, fileOffset, dst, 0, 32768);
			fileOffset += len;
			log.debug("Read " + len + " bytes over SFTP");
			
			// Get just the bytes we care about
			if ((len < 32768) && (len != -1)) {
				// Get only the correct
				byte[] temp = new byte[len];
				System.arraycopy(dst, 0, temp, 0, len);
				sbFile.append(new String(temp));
			} else {
				// Get everything in dst
				sbFile.append(new String(dst));
			}
			if (len < 32768) {
				break;
			}
		}
		
		// Cleanup
		client.closeFile(handle);
		return sbFile.toString();
	}
	
	/**
	 * 
	 * @param client
	 * @return
	 * @throws IOException
	 */
	private String cmd_overwrite(SFTPv3Client client, SFTPStepSettings settings) throws IOException {
		String path = settings.getPath();
		String filename = settings.getFilename();
		String content = settings.getContent();
		
		// Create the handle
		SFTPv3FileHandle handle = client.openFileRW(path + "/" + filename);
		
		// Write the file
		client.write(handle, 0, content.getBytes(), 0, content.length());
		
		// Cleanup
		client.closeFile(handle);
		
		return path + "/" + filename + " overwritten.";
	}
	
	/**
	 * 
	 * @param client
	 * @return
	 * @throws IOException
	 */
	private String cmd_forcewrite(SFTPv3Client client, SFTPStepSettings settings) throws IOException {
		String msg;
		try {
			// First try to create/write the file
			msg = cmd_create(client, settings);
		}
		catch(IOException createIOE) {
			// If the file already exists just overwrite
			msg = cmd_overwrite(client, settings);					
		}
		return msg;
	}
	
	/**
	 * 
	 * @param client
	 * @return
	 * @throws IOException
	 */
	private String cmd_create(SFTPv3Client client, SFTPStepSettings settings) throws IOException {
		String path = settings.getPath();
		String filename = settings.getFilename();
		String content = settings.getContent();
		
		// Create the handle
		SFTPv3FileHandle handle = client.createFile(path + "/" + filename);
		
		// Write the file
		client.write(handle, 0, content.getBytes(), 0, content.length());
		
		// Cleanup
		client.closeFile(handle);
		return path + "/" + filename + " created.";
	}
	
	/**
	 * 
	 * @param client
	 * @return
	 * @throws IOException
	 */
	private String cmd_delete(SFTPv3Client client, SFTPStepSettings settings) throws IOException {
		String path = settings.getPath();
		String filename = settings.getFilename();
		
		// Delete the file
		try {
			client.rm(path + "/" + filename);
		} catch (IOException ioe) {
			return path + "/" + filename + " was not deleted!";
		}
		return path + "/" + filename + " deleted.";		
	}

	/**
	 * 
	 */
	public ParameterList getParameters() {

        StringBuffer paramBuffer = new StringBuffer();

        paramBuffer.append(HOSTNAME)
        		   .append(PARAM_DELIMITER)
        		   .append(USERNAME)
        		   .append(PARAM_DELIMITER)
        		   .append(PASSWORD)
        		   .append(PARAM_DELIMITER)
        		   .append(COMMAND)
        		   .append(PARAM_DELIMITER)
        		   .append(PATH)
        		   .append(PARAM_DELIMITER)
        		   .append(FILENAME)
        		   .append(PARAM_DELIMITER)
        		   .append(CONTENT)
        		   .append(PARAM_DELIMITER);

        ParameterList parameters = new ParameterList(paramBuffer.toString());
        return parameters;
	}

	public void initialize(TestCase test, Element node) throws TestDefException {
		log.debug("Trilead step initialize: start");
	}

}
