package com.itko.lisa.ext.trilead;

import com.itko.lisa.editor.CustomEditor;
import com.itko.lisa.editor.TestCaseInfo;
import com.itko.lisa.editor.TestNodeInfo;
import com.itko.lisa.gui.LisaGuiPanel;
import com.itko.lisa.gui.LisaPrefs;
import com.itko.lisa.gui.ViewResultMasterPanel;
import com.itko.lisa.test.TestExec;
import com.itko.lisa.test.TestRunException;
//import com.itko.lisa.utils.BrowseForStreamButton;
import com.itko.util.StrUtil;
import com.itko.util.swing.Borders;
import com.itko.util.swing.GridPanel;
import com.itko.util.swing.PrefillCombo;
import com.itko.util.swing.panels.ProcessingCallback;
import com.itko.util.swing.panels.ProcessingDialogCancelledException;
import com.trilead.ssh2.Connection;

import java.awt.*;
import javax.swing.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

public class SFTPStepEditor extends CustomEditor {

	private static final long serialVersionUID = -580321237042596823L;

	static protected Log log = LogFactory.getLog(SFTPStepEditor.class);
	
    private JPanel editor;
    private ViewResultMasterPanel results;

    private PrefillCombo usernameCB;
    private JPasswordField passwordCB;
    private PrefillCombo hostCB;
    private PrefillCombo portCB;
    private JComboBox protocolCB;
    
    private JComboBox sftpCommandCB;
    private JComboBox scpCommandCB;
    private PrefillCombo pathCB;
    private PrefillCombo filenameCB;
    private PrefillCombo contentCB;
    private JCheckBox saveToFileCB; 
    private PrefillCombo destFilePathCB;
    private JButton destFilePathButton;
    private JComboBox authCB;
    private PrefillCombo privateKeyCB;
    private JPasswordField privateKeyPassCB;

    //private NodeListDropdown onError;

    protected final String SUPPORTED_SFTP_CMDS_LIST[] = {
    		SFTPStep.CMD_CREATE,
    		SFTPStep.CMD_DELETE,
    		SFTPStep.CMD_FORCEWRITE,
    		SFTPStep.CMD_LS,
    		SFTPStep.CMD_OVERWRITE,
    		SFTPStep.CMD_READ
    };
    
    protected final String SUPPORTED_SCP_CMDS_LIST[] = {
    		SFTPStep.CMD_GET,
    		SFTPStep.CMD_PUT
    };
    
    protected final String PROTOCOL_LIST[] = {
    		SFTPStep.PROTOCOL_SFTP,
    		SFTPStep.PROTOCOL_SCP,
    		SFTPStep.PROTOCOL_FTP
    };
    
    private void setup()
    {
        setupEditor();
        setupResponseEditor();

        setLayout( new BorderLayout() );
        add( editor, BorderLayout.NORTH );
        add( results, BorderLayout.CENTER );
    }


    private void setupEditor()
    {
        usernameCB = new PrefillCombo(getClass().getName() + ".user", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        passwordCB = new JPasswordField();
        hostCB = new PrefillCombo(getClass().getName() + ".host", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        portCB = new PrefillCombo(getClass().getName() + ".port", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        pathCB = new PrefillCombo(getClass().getName() + ".path", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        
        privateKeyCB = new PrefillCombo(getClass().getName() + ".private", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        privateKeyCB.setEnabled(false);
        privateKeyPassCB = new JPasswordField();
        privateKeyPassCB.setEnabled(false);
        
        authCB = new JComboBox(TrileadConnectionFactory.AUTH_TYPES);
        authCB.setEditable(false);
        authCB.addActionListener( new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		if (authCB.getSelectedItem().equals(TrileadConnectionFactory.AUTH_PUBLIC_KEY)) {
        			privateKeyCB.setEnabled(true);
        			privateKeyPassCB.setEnabled(true);
        		} else {
        			privateKeyCB.setEnabled(false);
        			privateKeyPassCB.setEnabled(false);
        		}
        	}
        });

        
        protocolCB = new JComboBox(PROTOCOL_LIST);
        protocolCB.setSelectedItem(SFTPStep.PROTOCOL_SFTP);
        protocolCB.setEditable(false);
        protocolCB.addActionListener( new ActionListener() {
        	public void actionPerformed( ActionEvent event ) {
        		// Figure out which protocol they have selected
        		String protocol = protocolCB.getSelectedItem().toString();
        		if (protocol.equalsIgnoreCase(SFTPStep.PROTOCOL_SFTP)) {
        			sftpCommandCB.setEnabled(true);
        			scpCommandCB.setEnabled(false);
        		} else if (protocol.equalsIgnoreCase(SFTPStep.PROTOCOL_SCP)) {
        			sftpCommandCB.setEnabled(false);
        			scpCommandCB.setEnabled(true);
//        		} else if (protocol.equalsIgnoreCase(SFTPStep.PROTOCOL_FTP)) {
//        			TODO
        		}
        	}
        });
        
        filenameCB = new PrefillCombo(getClass().getName() + ".filename", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        filenameCB.setEnabled(false);
        
        contentCB = new PrefillCombo(getClass().getName() + ".content", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        contentCB.setEnabled(false);
        
        destFilePathCB = new PrefillCombo(getClass().getName() + ".destFile", null, null, LisaPrefs.getLisaPrefsObj(), true, this );
        destFilePathCB.setEnabled(false);
        
        destFilePathButton = new JButton( " ... " );
        destFilePathButton.setBorder( BorderFactory.createEtchedBorder() );
        destFilePathButton.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e )
            {
                doFileChooser( destFilePathCB );
            }
        });
        
        saveToFileCB = new JCheckBox();
        saveToFileCB.setEnabled(false);
        saveToFileCB.addActionListener( new ActionListener() {
        	public void actionPerformed( ActionEvent event )
        	{
        		if (saveToFileCB.isSelected()) {
        			destFilePathCB.setEnabled(true);
        			destFilePathButton.setEnabled(true);
        		} else {
        			destFilePathCB.setEnabled(false);
        			destFilePathButton.setEnabled(false);
        		}
        	}
        });
        
        sftpCommandCB = new JComboBox(SUPPORTED_SFTP_CMDS_LIST);
        sftpCommandCB.setSelectedItem(SFTPStep.CMD_LS);
        sftpCommandCB.setEditable(false);
        sftpCommandCB.addActionListener( new ActionListener() {
        	public void actionPerformed( ActionEvent event )
        	{
        		// Figure out which command they have selected
        		String command = sftpCommandCB.getSelectedItem().toString();
        		if ((command.equalsIgnoreCase(SFTPStep.CMD_CREATE)) ||
        				(command.equalsIgnoreCase(SFTPStep.CMD_FORCEWRITE)) ||
        				(command.equalsIgnoreCase(SFTPStep.CMD_OVERWRITE)) ) {
        			filenameCB.setEnabled(true);
        			contentCB.setEnabled(true);
        			saveToFileCB.setEnabled(false);
        			destFilePathCB.setEnabled(false);
        			destFilePathButton.setEnabled(false);
        		} else if (command.equalsIgnoreCase(SFTPStep.CMD_READ)) {
        			filenameCB.setEnabled(true);
        			contentCB.setEnabled(false);
        			saveToFileCB.setEnabled(true);
        			if (saveToFileCB.isSelected()) {
        				destFilePathCB.setEnabled(true);
        				destFilePathButton.setEnabled(true);
        			} else {
        				destFilePathCB.setEnabled(false);
            			destFilePathButton.setEnabled(false);
        			}
        		} else if (command.equalsIgnoreCase(SFTPStep.CMD_DELETE)) {
        			filenameCB.setEnabled(true);
        			contentCB.setEnabled(false);
        			saveToFileCB.setEnabled(false);
    				destFilePathCB.setEnabled(false);
        			destFilePathButton.setEnabled(false);
        		} else if (command.equalsIgnoreCase(SFTPStep.CMD_LS)) {
        			filenameCB.setEnabled(false);
        			contentCB.setEnabled(false);
        			saveToFileCB.setEnabled(false);
        			destFilePathCB.setEnabled(false);
        			destFilePathButton.setEnabled(false);
        		}
        	}
        });

        scpCommandCB = new JComboBox(SUPPORTED_SCP_CMDS_LIST);
        scpCommandCB.setSelectedItem(SFTPStep.CMD_LS);
        scpCommandCB.setEditable(false);
        scpCommandCB.setEnabled(false);
        scpCommandCB.addActionListener( new ActionListener() {
        	public void actionPerformed( ActionEvent event )
        	{
        		// Figure out which command they have selected
        		String command = scpCommandCB.getSelectedItem().toString();
        		if (command.equalsIgnoreCase(SFTPStep.CMD_PUT)) {
        			filenameCB.setEnabled(true);
        			contentCB.setEnabled(true);
        			saveToFileCB.setEnabled(false);
        			destFilePathCB.setEnabled(false);
        			destFilePathButton.setEnabled(false);
        		} else if (command.equalsIgnoreCase(SFTPStep.CMD_GET)) {
        			filenameCB.setEnabled(true);
        			contentCB.setEnabled(false);
        			saveToFileCB.setEnabled(true);
        			if (saveToFileCB.isSelected()) {
        				destFilePathCB.setEnabled(true);
        				destFilePathButton.setEnabled(true);
        			} else {
        				destFilePathCB.setEnabled(false);
            			destFilePathButton.setEnabled(false);
        			}
        		}
        	}
        });
        
        JButton testConnB = new JButton("Test Connection");
        testConnB.addActionListener( new ActionListener() {
        	public void actionPerformed(ActionEvent event) {
        		
        		// Just test the connection only
        		execute(false);
        	}
        });
        JPanel testConnHolder = new JPanel( new FlowLayout( FlowLayout.LEFT ) );
        testConnHolder.add( testConnB );
        
        JButton executeB = new JButton( "Test Command" );
        executeB.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent event ) {
            	
            	// Run the full execute
                execute(true);
            }
        });
        JPanel testExecButtonHolder = new JPanel( new FlowLayout( FlowLayout.LEFT ));
        testExecButtonHolder.add( executeB );

        GridPanel gpConnect = new GridPanel(8);
        gpConnect.add("Username: ", usernameCB.getComponent());
        gpConnect.add("Password: ", passwordCB);
        gpConnect.add("Host: ", hostCB.getComponent());
        gpConnect.add("Port: ", portCB.getComponent());
        gpConnect.add("SSH Authentication Scheme", authCB);
        gpConnect.add("SSH Private Key", privateKeyCB.getComponent());
        gpConnect.add("SSH Private Key Passphrase", privateKeyPassCB);
        gpConnect.add(testConnHolder);
        gpConnect.add(new JLabel("Version: " + TrileadConnectionFactory.STEP_VERSION));
        JPanel connectPanel = gpConnect.getPanel();
        connectPanel.setBorder( Borders.createGroupBox("Connection Information"));
        
        GridPanel gpData = new GridPanel(9);
        gpData.add("Protocol: ", protocolCB);
        gpData.add("FTP/SFTP Command: ", sftpCommandCB);
        gpData.add("SCP Command: ", scpCommandCB);
        gpData.add("Remote Path: ", pathCB.getComponent());
        gpData.add("Remote Filename: ", filenameCB.getComponent());
        
        gpData.add("Content: ", contentCB.getComponent());
        gpData.add("Save to file: ", saveToFileCB);
        gpData.add("Local File Path: ", destFilePathCB.getComponent(), destFilePathButton);

        gpData.add(testExecButtonHolder);
        JPanel dataPanel = gpData.getPanel();
        dataPanel.setBorder( Borders.createGroupBox("Command Information"));
        
        //editor = new JPanel( new BorderLayout() );
        editor = new JPanel( new GridLayout(1,2) );
        editor.setBorder( Borders.createGroupBox(null));
        editor.add(connectPanel);
        editor.add(dataPanel);
    }

    private void setupResponseEditor()
    {
        results = new ViewResultMasterPanel( false, true );
    }
    
    private void execute(final boolean bFullExecute)
    {
        final TestNodeInfo ni = (TestNodeInfo)getController();
        final TestCaseInfo tci = ni.getTestCaseInfo();
        final TestExec ts = tci.getTestExec();

        // Create the settings object
        final SFTPStepSettings settings = new SFTPStepSettings();
        settings.setUsername(usernameCB.getText() );
        settings.setPassword(new String(passwordCB.getPassword()) );
        settings.setHostname(hostCB.getText() );
        settings.setPort(portCB.getText() );
        settings.setProtocol(protocolCB.getSelectedItem().toString() );
        settings.setPath(pathCB.getText() );
        settings.setFilename(filenameCB.getText() );
        settings.setContent(contentCB.getText() );
        settings.setSaveToFile(saveToFileCB.isSelected());
        settings.setDestFile(destFilePathCB.getText());
        settings.setAuth(authCB.getSelectedItem().toString());
        settings.setPrivateKey(privateKeyCB.getText());
        settings.setPrivateKeyPass(new String(privateKeyPassCB.getPassword()));
        
        // Depending on the protocol, pick the correct command
        if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_SFTP)) {
        	settings.setCommand(sftpCommandCB.getSelectedItem().toString() );
        } else if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_SCP)) {
        	settings.setCommand(scpCommandCB.getSelectedItem().toString() );
        } else if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_FTP)) {
        	settings.setCommand(sftpCommandCB.getSelectedItem().toString() );
        }
        
        String msg = isEditorValid();
        if( msg != null ) {
            LisaGuiPanel.infoMsg( this, getName(), msg );
            return;
        }

        final StringBuffer sb = new StringBuffer();
        try {
            LisaGuiPanel.doProcessingDialog( getName(), this,
                "Doing SFTP",
                new ProcessingCallback() {
                    public void doCallback( Object o ) throws Throwable
                    {
                    	if (bFullExecute) {
	                    	
                    		// Perform a full execute
                    		String msg = "Unknown protocol.";
	                    	if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_SFTP)) {
	                    		SFTPEngine engine = SFTPStep.getSFTPEngine();
	                    		msg = engine.execute(settings, ts);
	                    		sb.append("SFTP Command completed successfully");
	                    	} else if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_SCP)) {
	            				SCPEngine engine = SFTPStep.getSCPEngine();
	            				msg = engine.execute(settings, ts);
	                    		sb.append("SCP Command completed successfully");
	            			} else if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_FTP)) {
	            				FTPEngine engine = SFTPStep.getFTPEngine();
	            				msg = engine.execute(settings, ts);
	            				sb.append("FTP Command completed successfully");
	            			} else {
	            				sb.append("Unknown protocol: " + settings.getProtocol());
	            			}
	                    	ni.setRet(msg);
                    	} else {
                    		
                    		// Just test the connection only
                    		String tsHostname = ts.parseInState(settings.getHostname());
                    		String tsPort = ts.parseInState(settings.getPort());
                    		String tsUsername = ts.parseInState(settings.getUsername());
                    		String tsPassword = ts.parseInState(settings.getPassword());
                    		String tsAuth = ts.parseInState(settings.getAuth());
                    		String tsPrivateKey = ts.parseInState(settings.getPrivateKey());
                    		String tsPrivateKeyPass = ts.parseInState(settings.getPrivateKeyPass());
                    		
                    		Connection conn = TrileadConnectionFactory.getConnection(tsHostname, tsUsername, tsPassword, tsAuth, tsPrivateKey, tsPrivateKeyPass);
                    		boolean bPassed = TrileadConnectionFactory.testConnection(conn);
                    		if (bPassed) {
                    			sb.append("Connection Test Passed.");
                    			TrileadConnectionFactory.cleanupConnection(conn, tsHostname, tsUsername, tsAuth, false);
                    		} else {
                    			sb.append("Connection Test Failed.");
                    			TrileadConnectionFactory.cleanupConnection(conn, tsHostname, tsUsername, tsAuth, true);
                    		}
                    	}
                    }
                }, true );
            
            // Popup dialog with response
            LisaGuiPanel.infoMsg( this, getName(), sb.toString() );
        }
        catch( ProcessingDialogCancelledException can )
        {
            log.info( "You gave up, but the SFTP will continue until it completes in the background." );
        }
        catch( Throwable tw )
        {
            TestRunException tre = new TestRunException("Error executing SFTP: " + " " + tw.getMessage(), null);
            LisaGuiPanel.errorMsg( this, getName(), "Error executing SFTP: " + " " + tw.getMessage(), tre );
        }

        results.setNodeInfo( tci, ni );
    }
    
    public void display()
    {
        if( usernameCB == null )
            setup();

        TestNodeInfo ni = (TestNodeInfo)getController();
        SFTPStep sftp = (SFTPStep)ni.getAttribute(  SFTPStepController.SFTP_KEY );
        SFTPStepSettings settings = sftp.getSettings();
        if (settings != null) {
	        usernameCB.setText( settings.getUsername() );
	        passwordCB.setText( settings.getPassword() );
	        hostCB.setText( settings.getHostname() );
	        portCB.setText( settings.getPort() );
	        pathCB.setText( settings.getPath() );
	        protocolCB.setSelectedItem( settings.getProtocol() );
	        filenameCB.setText( settings.getFilename() );
	        contentCB.setText( settings.getContent() );
	        saveToFileCB.setSelected( settings.isSaveToFile() );
	        destFilePathCB.setText( settings.getDestFile() );
	        if (settings.getAuth() == null) {
	        	authCB.setSelectedItem(TrileadConnectionFactory.AUTH_PASSWORD);
	        } else {
	        	authCB.setSelectedItem( settings.getAuth() );
	        }
	        privateKeyCB.setText( settings.getPrivateKey() );
	        privateKeyPassCB.setText( settings.getPrivateKeyPass() );
	        
	        // Depending on the protocol, pick the correct command
	        if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_SFTP)) {
	        	sftpCommandCB.setSelectedItem( settings.getCommand() );
	        } else if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_SCP)) {
	        	scpCommandCB.setSelectedItem( settings.getCommand() );
	        } else if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_FTP)) {
	        	sftpCommandCB.setSelectedItem( settings.getCommand() );
	        }
        }
        results.setNodeInfo( ni.getTestCaseInfo(), ni );
    }


    public String isEditorValid()
    {
        if( StrUtil.isEmpty( hostCB.getText() ) )
            return "Need a host";

//        if( StrUtil.isEmpty( pathCB.getText() ) )
//            return "Need a source path";
        
        return null;
    }


    public void save()
    {
        TestNodeInfo ni = (TestNodeInfo)getController();
        ni.getTestCaseInfo().getTestExec().saveNodeResponse( ni.getName(), ni.getRet() );

        SFTPStep sftp = (SFTPStep)ni.getAttribute( SFTPStepController.SFTP_KEY );
        
        SFTPStepSettings settings = new SFTPStepSettings();
        settings.setUsername( usernameCB.getText() );
        settings.setPassword( new String(passwordCB.getPassword()));
        settings.setHostname( hostCB.getText() );
        settings.setPort( portCB.getText() );
        settings.setPath( pathCB.getText() );
        settings.setFilename( filenameCB.getText() );
        settings.setProtocol( protocolCB.getSelectedItem().toString() );
        settings.setContent( contentCB.getText() );
        settings.setSaveToFile( saveToFileCB.isSelected() );
        settings.setDestFile(destFilePathCB.getText());
        settings.setAuth( authCB.getSelectedItem().toString() );
        settings.setPrivateKey( privateKeyCB.getText() );
        settings.setPrivateKeyPass( new String(privateKeyPassCB.getPassword()) );

        // Depending on the protocol, pick the correct command
        if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_SFTP)) {
        	settings.setCommand(sftpCommandCB.getSelectedItem().toString() );
        } else if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_SCP)) {
        	settings.setCommand(scpCommandCB.getSelectedItem().toString() );
        } else if (settings.getProtocol().equalsIgnoreCase(SFTPStep.PROTOCOL_FTP)) {
        	settings.setCommand(sftpCommandCB.getSelectedItem().toString() );
        }
        
        sftp.setSettings(settings);
    }

    private void doFileChooser( PrefillCombo cb )
    {
        try {
            JFileChooser chooser;
            String cur = cb.getText();

            if( cur != null && cur.length() > 0 )
                chooser = new JFileChooser( cur );
            else
                chooser = new JFileChooser();

            chooser.setApproveButtonText( "Select" );

            int option;

            option = chooser.showOpenDialog( this );

            if( option == JFileChooser.APPROVE_OPTION ) {
                File shownFile = chooser.getSelectedFile();
//                cb.setText(BrowseForStreamButton.convertPathToPropertyString(shownFile.getCanonicalPath() ));
                cb.setText(shownFile.getCanonicalPath());
            }
        } catch (IOException e) {
            // do nothing by design
        }
    }
}
