package com.itko.lisa.ext.trilead;

import java.io.PrintWriter;
import java.util.Collection;

import com.itko.lisa.editor.TestNodeInfo;
import com.itko.lisa.test.StepConnection;
import com.itko.lisa.test.StepConnectionType;
import com.itko.lisa.test.StepNameChangeListener;
import com.itko.lisa.resources.Strings;

public class SSHStepController extends TestNodeInfo implements StepNameChangeListener {

    static final String SSH_STEP_KEY = "lisa.sshstep.key";
    
	@Override
	public void initNewOne() {
        SSHStep ssh = new SSHStep();
        putAttribute( SSH_STEP_KEY, ssh );
	}

	@Override
	public String loadSmallIcon()
    {
//        return "ssh_16.png";
    	return Strings.get("icon.tctree.ftpnode", "doc.gif");
    }

	@Override
	public String loadLargeIcon()
    {
//        return "ssh_64.png";
        return Strings.get("icon.tctree.ftpnode.lg", "doc.gif");
    }

	@Override
	public void writeSubXML(PrintWriter pw) {
        SSHStep ssh = (SSHStep)getAttribute( SSH_STEP_KEY );
        ssh.writeSubXML( pw );
	}

	@Override
	public String getEditorName() {
		return "SSH";
	}

	@Override
	public void migrate(Object testCaseObj) {
        SSHStep ssh = (SSHStep)testCaseObj;
        putAttribute( SSH_STEP_KEY, ssh );

	}

	public String getHelpString() {
		return "This step is used to create SSH requests to a remote server";
	}


    /**
     * This method should be overridden by subclasses to provide any step connections beyond
     * the standard "next" step connection.  By default, this method does nothing.
     *
     * @param stepConnections the collection to add new connections to.
     */	
    @Override public void gatherOtherConnections(Collection<StepConnection> stepConnections)
    {
        SSHStep ssh = (SSHStep)getAttribute( SSH_STEP_KEY );
        String onException = ssh == null ? null : ssh.getErrorNode();

        StepConnection.onError(getName(), onException, this, stepConnections);
    }

    /**
     * This method is used to inform the step controller that a secondary step connection needs
     * to be updated.  The framework will handle notifying any related editor as appropriate.
     *
     * @param connetion the connection whose target step is being changed.
     * @param newTargetStep the name of the new target step for the connection.
     */
    @Override public void secondaryStepReferenceChanged(StepConnection connection, String newTargetStep)
    {
        SSHStep step = (SSHStep)getAttribute( SSH_STEP_KEY );

        if (step != null && connection.getType() == StepConnectionType.ON_ERROR &&
            connection.getSourceStep().equals(getName()))
            step.setErrorNode(newTargetStep);
    }

    /**
     * This method is used to inform the implementor that the name of a step has been changed.
     *
     * @param oldName the original name of the step.
     * @param newName the new name of the step.
     */
    public void stepNameChanged(String oldName, String newName)
    {
        SSHStep ssh = (SSHStep)getAttribute( SSH_STEP_KEY );
        String onException = ssh == null ? null : ssh.getErrorNode();

        if (oldName.equals(onException))
            ssh.setErrorNode(newName);
    }
}
