package com.itko.lisa.ext.trilead.xterm;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.itko.lisa.test.LisaException;
import com.trilead.ssh2.Session;

public class XtermEngine extends Thread implements Serializable {

	private static final long serialVersionUID = -4440427733017195597L;
	
	public static final int STATUS_TERMINAL = 0;	// Terminal enabled only
	public static final int STATUS_RECORDING = 1;	// For recording
	public static final int STATUS_PLAYBACK = 2;	// For playback

	
	static protected Log log = LogFactory.getLog(XtermEngine.class);
	private static Map<String, XtermEngine> engineMap = new HashMap<String, XtermEngine>();
	
	private int iEngineStatus;
	private String key;
	private Session sess;
	private BufferedReader in;
	private OutputStream out;
	private XtermUI ui;
	
	public static XtermEngine getEngine(String tsHostname, String tsUsername, String tsAuth) {
		
		// Try to find their XtermEngine in the HashMap
		String key = tsUsername + "@" + tsHostname + ":" + tsAuth;
		XtermEngine engine = engineMap.get(key);
		
		// If there was nothing in the HashMap, make them a new engine
		if (engine == null) {
			log.debug("XtermEngine was not found in the map, making a new one");
			engine = new XtermEngine(key);
			engineMap.put(key, engine);
		}
		
		return engine;
	}
	
	public XtermEngine(String key) {
		this.key = key;
	}
	
	public void connect(Session sess) throws IOException {
		this.sess = sess;
		
		sess.requestPTY("dumb", XtermUI.DEF_WIDTH, XtermUI.DEF_HEIGHT, 0, 0, null);
		sess.startShell();
		in = new BufferedReader(new InputStreamReader(sess.getStdout()));
		out = sess.getStdin();
	}
	
	/**
	 * Write this character out the output stream, and possibly record it
	 * @param key
	 */
	public void write(char key) {
		try
		{
			out.write(key);
		}
		catch (IOException ioe)
		{
			// eat
		}
	}
	
	public void logout() {
		sess.close();
	}
	
	/**
	 * Start the thread
	 */
	public void run() {
		// Start working with the connection
		char[] cbuf = new char[8192];
		int len;
		log.debug("Starting reader thread");
		try
		{
			while ((len = in.read(cbuf)) != -1)
			{
				// Go one character at a time
				for (int i = 0; i < len; i++) {
					// Look for backspace character
					if ((cbuf[i] & 0xff) == 8) {
						ui.backspace();
					} else {
						// Otherwise print the character
						String ch = String.valueOf(cbuf[i]);
						ui.appendScreen(ch);
					}
				}
			}
			log.debug("The thread has stopped running, len = " + len);
		}
		catch (Exception e)
		{
			String str = LisaException.exceptionToString( "XtermEngine Error: ", "XtermEngine", e.getMessage(), e );
			log.error("XtermEngine Exception: " + e.getMessage());
			log.error(str);
		}
		finally
		{
			// Take this thread out of the HashMap
			log.debug("XtermEngine is taken out of the map");
			engineMap.remove(key);
		}
	}
	
	public void setUI(XtermUI ui) {
		this.ui = ui;
	}
	
	public void setEngineStatus(int iEngineStatus) {
		this.iEngineStatus = iEngineStatus;
	}
}
