package com.itko.lisa.ext.trilead.xterm;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.WindowConstants;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.itko.lisa.editor.TestCaseInfo;
import com.itko.lisa.editor.TestNodeInfo;
import com.itko.lisa.ext.trilead.SSHStep;
import com.itko.lisa.ext.trilead.SSHStepController;
import com.itko.lisa.gui.ListTable;
import com.itko.lisa.gui.ListTable.ListTableModel;
import com.itko.lisa.test.TestDefException;
import com.itko.util.swing.GridPanel;
import com.trilead.ssh2.Session;

public class XtermUI extends JDialog implements ActionListener, WindowListener, KeyListener {
	
	private static final long serialVersionUID = -4806255641635018753L;
	static protected Log log = LogFactory.getLog(XtermUI.class);
	public static final int DEF_HEIGHT = 24;
	public static final int DEF_WIDTH = 80;
	public static final String REC_START = "Start Recording";
	public static final String REC_STOP = "Stop Recording";
	
	
	
//	private boolean isRecording;
	private boolean isExpectFinished;
	private int iTerminalStatus;
	private List<StringBuffer> recCmdList;
	private StringBuffer lastCmd;
	private List<StringBuffer> recExpectList;
	private StringBuffer lastExpect;
	private XtermEngine engine;
	private JTabbedPane tabbedPane;
	private JTextArea screen;	
	private JButton recB;
	private JButton closeB;
	private JButton addB;
	private JLabel lblExpectCount;
	private ListTable expectTbl;
	private ListTableModel expectTM;
	
	private TestCaseInfo tci;
	
	public XtermUI(JFrame owner) {
		
		// Start a modal terminal window
		super(owner, "LISA Terminal", true);
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(this);

		// Create the UI screen
		screen = new JTextArea(DEF_HEIGHT, DEF_WIDTH);
		screen.setFont(new Font("Courier New", Font.PLAIN, 16));
		screen.setBackground(Color.BLACK);
		screen.setForeground(Color.WHITE);
		screen.setCaretColor(Color.WHITE);
		screen.setEditable(false);
		
		// Create the key listener
		screen.addKeyListener(this);
		
		// Make the scroll pane to hold the screen
		JScrollPane scrollPane = new JScrollPane(screen,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		
		// Create the table
        // Create the column list
        List<String> cols = new ArrayList<String>();
        cols.add("Trans #");
        cols.add("Expect Match Regexp");
        cols.add("Command");
        expectTbl = new ListTable(cols, null);
        expectTM = expectTbl.getTableModel();
		
		// Make the panel
		GridPanel gpExpect = new GridPanel(2);
		lblExpectCount = new JLabel("0");
		gpExpect.add("Commands Recorded: ", lblExpectCount);
		gpExpect.add(expectTbl);
		
		// Create a tabbed pane to hold it all
		tabbedPane = new JTabbedPane();
		tabbedPane.addTab("LISA Terminal", scrollPane);
		tabbedPane.addTab("Recorded Elements", gpExpect.getPanel());
		getContentPane().add(tabbedPane, BorderLayout.CENTER);
		
		// Create the Buttons
		recB = new JButton("Start Recording");
		recB.addActionListener(this);
		addB = new JButton("Add to Test Case and Close");
		addB.addActionListener(this);
		addB.setEnabled(false);
		closeB = new JButton("Cancel and Close");
		closeB.addActionListener(this);
		
		// Layout the buttons
		JPanel buttonPanel = new JPanel(new FlowLayout());
		buttonPanel.add(recB);
		buttonPanel.add(addB);
		buttonPanel.add(closeB);
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		
		setResizable(false);
		pack();
		setLocationRelativeTo(null);
	}
	
	public void loadEngine(String tsHostname, String tsUsername, String tsAuth, Session sess) throws IOException {
		
		// First locate the engine
		XtermEngine engine = XtermEngine.getEngine(tsHostname, tsUsername, tsAuth);
		
		// Setup the engine
		engine.setUI(this);
		engine.connect(sess);
		this.engine = engine;
	}
	
	public void startEngine(int iTerminalStatus, TestCaseInfo tci) {
		// Reset the command list
		setTerminalStatus(iTerminalStatus);
		if (iTerminalStatus == XtermEngine.STATUS_RECORDING) {
			this.tci = tci;
			recCmdList = new ArrayList<StringBuffer>();
			lastCmd = new StringBuffer("");
			recExpectList = new ArrayList<StringBuffer>();
			lastExpect = new StringBuffer("");
		}
		
		// Reset the screen
		screen.setText("");
		
		// Kick off the engine thread
		if (!engine.isAlive()) {
			engine.start();
		}
		
	}
	
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source.equals(recB)) {
			// Toggle the recording
			if (recB.getText().equals(REC_START)) {
				// Show the terminal view
//				setIsRecording(true);
				setTerminalStatus(XtermEngine.STATUS_RECORDING);
				tabbedPane.setSelectedIndex(0);
				addB.setEnabled(false);
			} else {
				// Show the results view
//				setIsRecording(false);
				setTerminalStatus(XtermEngine.STATUS_TERMINAL);
				tabbedPane.setSelectedIndex(1);
				addB.setEnabled(true);
			}
		} else if (source.equals(addB)) {
			
			// Create a new SSH node
			SSHStep sshNode = new SSHStep();
			sshNode.setName("New SSH Step");
			
			// Create a new node
			SSHStepController sshController = new SSHStepController();
			try {
				sshController.migrate(sshNode, tci);
				tci.addNode(sshController, -1);
				sshController.setIsNewTMCreated(true);
			} catch (TestDefException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
			// Close
			engine.logout();
			this.setVisible(false);
			
		} else if (source.equals(closeB)) {
			
			// TODO: Cancel and close
			engine.logout();
			this.setVisible(false);
		}
	}
	
	public void appendScreen(String text) {
		screen.append(text);
		screen.setCaretPosition(screen.getDocument().getLength());
		
		// Add this to the lastExpect buffer
		// Expect not finished if the user hasn't typed yet
		if (!isExpectFinished) {
			lastExpect.append(text);
		}
	}
	
	public void backspace() {
		// Take the last character out of the textArea
		int end = screen.getDocument().getLength();
		screen.replaceRange(null, (end-1), end);
		screen.setCaretPosition(screen.getDocument().getLength());
		
		// Take the last character out of the lasCmd (used when recording)
//		if (isRecording) {
		if (iTerminalStatus == XtermEngine.STATUS_RECORDING) {
			end = lastCmd.length();
			lastCmd.deleteCharAt(end-1);
		}
	}
	
	/**
	 * Set whether the lisaTerminal is in recording mode.
	 * @param isRecording
	 */
	public void setTerminalStatus(int iTerminalStatus) {
		this.iTerminalStatus = iTerminalStatus;
		this.isExpectFinished = false;
		
		// Update the button text
		if (iTerminalStatus == XtermEngine.STATUS_RECORDING) {
			recB.setText(REC_STOP);
		} else {
			recB.setText(REC_START);
		}
		
		// Tell the engine whether or not to record
		if (engine != null) {
			engine.setEngineStatus(iTerminalStatus);
		}
	}

	public void windowActivated(WindowEvent e) {}

	public void windowClosed(WindowEvent e) {}

	public void windowClosing(WindowEvent e) {
		
//		if (isRecording) {
		if (iTerminalStatus == XtermEngine.STATUS_RECORDING) {
			for (int i = 0; i < recCmdList.size(); i++) {
				StringBuffer sbCmd = recCmdList.get(i);
				StringBuffer sbExpect = recExpectList.get(i);
				
				// 
				String cmd = sbCmd.toString();
				String expect = sbExpect.substring(sbExpect.lastIndexOf("\n"));
				
				log.debug("Expect: " + expect + " // Command: " + cmd);
			}
		}
		
		engine.logout();
		this.setVisible(false);
	}

	public void windowDeactivated(WindowEvent e) {}

	public void windowDeiconified(WindowEvent e) {}

	public void windowIconified(WindowEvent e) {}

	public void windowOpened(WindowEvent e) {}

	public void keyPressed(KeyEvent e) {}

	public void keyReleased(KeyEvent e) {}

	public void keyTyped(KeyEvent e) {
		char key = e.getKeyChar();
		engine.write(key);
		e.consume();
		
//		if (isRecording) {
		if (iTerminalStatus == XtermEngine.STATUS_RECORDING) {
			
			// Look for the carriage-return
			if (key == '\n') {
				
				// Add this command
				recCmdList.add(lastCmd);
				
				// Add this expect
				String expect = lastExpect.substring(lastExpect.lastIndexOf("\n")).trim();
				
				// Update the count label
				lblExpectCount.setText(recCmdList.size() + "");
				
				// Update the table model
				int txCount = recCmdList.size();
		    	List<String> row = new ArrayList<String>(3);
		    	row.add(txCount + "");
				row.add(".*" + expect);
				row.add(lastCmd.toString());
				expectTM.addRow(row);
				
				// Reset
				lastCmd = new StringBuffer("");
				lastExpect = new StringBuffer("");
				isExpectFinished = false;
				
			} else {
				
				// If lastCmd is empty this is the start of a new command
				lastCmd.append(key);
				isExpectFinished = true;
			}
		}
	}
}
