package com.itko.lisa.remote.vse.ext;

import com.google.common.base.Function;
import com.itko.lisa.remote.utils.Log;
import com.itko.lisa.remote.vse.AbstractVSEInterceptor;
import com.itko.lisa.remote.vse.VSEFrame;

public class GXSDispatchingVSEInterceptor extends AbstractVSEInterceptor
{

    /** Returns true iff this call should not be intercepted */
    public boolean onPostRecord( final VSEFrame frame,
                                 final Object src,
                                 final String clazz,
                                 final String method,
                                 final String signature,
                                 final Object[] args,
                                 final Object ret )
    {
        Function<Object, Boolean> superCall = new Function<Object, Boolean>()
        {
            @Override
            public Boolean apply( final Object input )
            {
                return GXSDispatchingVSEInterceptor.super.onPostRecord( frame,
                                                                        src,
                                                                        clazz,
                                                                        method,
                                                                        signature,
                                                                        args,
                                                                        ret );
            }
        };

        if( "gxs.iefecsr.ejbsend.ICSSendSessionHelper".equals( clazz ) )
        {
            return ICSSendSessionHelperInterceptor.onPostRecord( frame,
                                                                 method,
                                                                 args,
                                                                 superCall );
        }
        return superCall.apply( null );
    }

    /** Returns the result to override with or com.itko.lisa.remote.instrument.Rules.PROCEED to keep going with VSE */
    public Object onPreHijack( final VSEFrame frame,
                               final Object src,
                               final String clazz,
                               final String method,
                               final String signature,
                               final Object[] args,
                               final Object ret )
    {
        Function<Object, Object> superCall = new Function<Object, Object>()
        {
            @Override
            public Object apply( final Object input )
            {
                return GXSDispatchingVSEInterceptor.super.onPreHijack( frame,
                                                                       src,
                                                                       clazz,
                                                                       method,
                                                                       signature,
                                                                       args,
                                                                       ret );
            }
        };

        if( "gxs.iefecsr.ejbsend.ICSSendSessionHelper".equals( clazz ) )
        {
            return ICSSendSessionHelperInterceptor.onPreHijack( frame,
                                                                method,
                                                                args,
                                                                superCall );
        }

        return superCall.apply( null );
    }

    /**
     * Returns the result to override with or com.itko.lisa.remote.instrument.Rules.PROCEED to keep going with VSE's
     * response
     */
    public Object onPostHijack( final VSEFrame frame,
                                final Object src,
                                final String clazz,
                                final String method,
                                final String signature,
                                final Object[] args,
                                final Object ret )
    {
        Function<Object, Object> superCall = new Function<Object, Object>()
        {
            @Override
            public Object apply( final Object input )
            {
                return GXSDispatchingVSEInterceptor.super.onPostHijack( frame,
                                                                        src,
                                                                        clazz,
                                                                        method,
                                                                        signature,
                                                                        args,
                                                                        ret );
            }
        };

        if( "gxs.iefecsr.ejbreceive.ICSReceiveSessionHelper".equals( clazz ) )
        {
            return ICSReceiveSessionHelperInterceptor.onPostHijack( method,
                                                                    args,
                                                                    superCall );
        }

        return superCall.apply( null );
    }
}