package com.itko.lisa.remote.vse.ext;

import com.google.common.base.Function;
import com.itko.lisa.remote.utils.Log;
import com.itko.lisa.remote.vse.VSEFrame;

import java.io.InputStream;
import java.lang.reflect.Method;

public class ICSSendSessionHelperInterceptor
{

    /**
     * When recording, read the contents of the InputStream and stick it into a
     * string on the frame's arguments before sending it to VSE
     */
    public static boolean onPostRecord( VSEFrame frame,
                                        String method,
                                        Object[] args,
                                        Function<Object, Boolean> superCall )
    {
        if( "doTrustedSendSession".equals( method ) &&
            args[5] instanceof InputStream )
        {
            overrideArguments( frame, args );
        }
        return superCall.apply( null );
    }

    /**
     * When playing back, read the contents of the InputStream and stick it into
     * as string on the frame's arguments so that VSE can match correctly
     */
    public static Object onPreHijack( VSEFrame frame,
                                      String method,
                                      Object[] args,
                                      Function<Object, Object> superCall )
    {
        if( "doTrustedSendSession".equals( method ) &&
            args[5] instanceof InputStream )
        {
            overrideArguments( frame, args );
        }
        return superCall.apply( null );
    }

    /** Reset the last arg from an InputStream to its stringified content */
    private static void overrideArguments( VSEFrame frame, Object[] args )
    {
        try
        {
            InputStream inputStream = (InputStream) args[5];
            String content = new String();
            int nBytesRead = 0;
            byte[] barrData = new byte[10000];
            if( inputStream.markSupported() )
            {
                inputStream.reset();
            }
            while( (nBytesRead = inputStream.read( barrData, 0, barrData.length )) != -1 )
            {
                content = content + new String( barrData, 0, nBytesRead );
            }

            args[5] = content;
            // Unfortunately setArguments is not public so let's use
            // reflection here
            Method setArgs = VSEFrame.class.getDeclaredMethod( "setArguments",
                                                               new Class[] {Object[].class} );
            setArgs.setAccessible( true );
            setArgs.invoke( frame, new Object[] {args} );
        }
        catch( Exception e )
        {
            Log.getLogger().warn( "Oops, can't transform stream: " + e );
        }
    }
}