package com.itko.lisa.remote.vse.ext;

import com.itko.lisa.remote.instrument.Rules;
import com.itko.lisa.remote.utils.Log;
import com.itko.lisa.remote.vse.AbstractVSEInterceptor;
import com.itko.lisa.remote.vse.VSEFrame;

public class GXSDispatchingVSEInterceptor extends AbstractVSEInterceptor
{

    /** Returns true iff this call should not be intercepted */
    public boolean onPostRecord( final VSEFrame frame,
                                 final Object src,
                                 final String clazz,
                                 final String method,
                                 final String signature,
                                 final Object[] args,
                                 final Object ret )
    {
        Log.getLogger().debug( "Handling onPostRecord for method '" + method + "'" );

        ParentCaller parentCaller = new ParentCaller( false );
        boolean subCallReturn = false;
        boolean subCalled = false;

        if( "gxs.iefecsr.ejbsend.ICSSendSessionHelper".equals( clazz ) )
        {
            Log.getLogger().debug( "Recognized '" + clazz + "', dispatching" );
            subCallReturn = ICSSendSessionHelperInterceptor.onPostRecord( frame,
                                                                          method,
                                                                          signature,
                                                                          args,
                                                                          parentCaller );
            subCalled = true;
        }

        if( !subCalled || parentCaller.callParent() )
        {
            return GXSDispatchingVSEInterceptor.super.onPostRecord( frame,
                                                                    src,
                                                                    clazz,
                                                                    method,
                                                                    signature,
                                                                    args,
                                                                    ret );
        }
        else
        {
            return subCallReturn;
        }
    }

    /** Returns the result to override with or com.itko.lisa.remote.instrument.Rules.PROCEED to keep going with VSE */
    public Object onPreHijack( final VSEFrame frame,
                               final Object src,
                               final String clazz,
                               final String method,
                               final String signature,
                               final Object[] args,
                               final Object ret )
    {
        Log.getLogger().debug( "Handling onPreHijack for method '" + method + "'" );

        ParentCaller parentCaller = new ParentCaller( false );
        Object subCallReturn = Rules.PROCEED;
        boolean subCalled = false;

        if( "gxs.iefecsr.ejbsend.ICSSendSessionHelper".equals( clazz ) )
        {
            Log.getLogger().debug( "Recognized '" + clazz + "', dispatching" );
            subCallReturn = ICSSendSessionHelperInterceptor.onPreHijack( frame,
                                                                         method,
                                                                         signature,
                                                                         args,
                                                                         parentCaller );
            subCalled = true;
        }

        if( !subCalled || parentCaller.callParent() )
        {
            return GXSDispatchingVSEInterceptor.super.onPreHijack( frame,
                                                                   src,
                                                                   clazz,
                                                                   method,
                                                                   signature,
                                                                   args,
                                                                   ret );
        }
        else
        {
            return subCallReturn;
        }
    }

    /**
     * Returns the result to override with or com.itko.lisa.remote.instrument.Rules.PROCEED to keep going with VSE's
     * response
     */
    public Object onPostHijack( final VSEFrame frame,
                                final Object src,
                                final String clazz,
                                final String method,
                                final String signature,
                                final Object[] args,
                                final Object ret )
    {
        Log.getLogger().debug( "Handling onPostHijack for method '" + method + "'" );

        ParentCaller parentCaller = new ParentCaller( false );
        Object subCallReturn = Rules.PROCEED;
        boolean subCalled = false;

        if( "gxs.iefecsr.ejbreceive.ICSReceiveSessionHelper".equals( clazz ) )
        {
            Log.getLogger().debug( "Recognized '" + clazz + "', dispatching" );
            subCallReturn = ICSReceiveSessionHelperInterceptor.onPostHijack( method,
                                                                             signature,
                                                                             args,
                                                                             parentCaller );
            subCalled = true;
        }

        if( !subCalled || parentCaller.callParent() )
        {
            return GXSDispatchingVSEInterceptor.super.onPostHijack( frame,
                                                                    src,
                                                                    clazz,
                                                                    method,
                                                                    signature,
                                                                    args,
                                                                    ret );
        }
        else
        {
            return subCallReturn;
        }
    }

    protected class ParentCaller
    {
        boolean callParent;

        public ParentCaller( final boolean callParent )
        {
            this.callParent = callParent;
        }

        public boolean callParent()
        {
            return callParent;
        }

        public void setCallParent( final boolean callParent )
        {
            this.callParent = callParent;
        }
    }
}