package com.itko.lisa.remote.vse.ext;

import com.itko.lisa.remote.Agent;
import com.itko.lisa.remote.instrument.Rules;
import com.itko.lisa.remote.utils.Log;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class ICSReceiveSessionHelperInterceptor
{
//    private static final Log log = LogFactory.getLog( ICSReceiveSessionHelperInterceptor.class );

    /** Returns the result to override with or null to keep going with VSE */
    public static Object onPostHijack( String method,
                                       String signature,
                                       Object[] args,
                                       GXSDispatchingVSEInterceptor.ParentCaller superCall )
    {
        Log.getLogger().debug( "In onPostHijack of ICSReceiveSessionHelperInterceptor" );

        if( "doReceiveSession".equals( method ) &&
            args[5] instanceof OutputStream &&
            args[2] instanceof Long )
        {
            Log.getLogger().debug( "Handling " + method );
            OutputStream os = (OutputStream) args[5];
            long iLog = ((Long) args[2]).longValue();

            Agent agent = Agent.getInstance();
            String path = agent.getAgentDir() + "/" + "agentTxfr-" + iLog + ".tmp";
            File file = new File( path );

            try
            {
                Log.getLogger().debug( "Reading from file at " + path + " and writing to OutputStream" );
                byte[] fileBytes = toByteArray( file );

                os.write( fileBytes );
                if( file.delete() ) {} //Don't care
            }
            catch( IOException ioe )
            {
                // Om,nom,nom
                Log.getLogger().error( ioe.getMessage(), ioe );
            }
        }
        else if( Log.getLogger().isDebugEnabled() )
        {
            Log.getLogger().debug( "Not an expected method.  Saw " + method + " with signature:" + signature + "." );
        }

        superCall.setCallParent( true );

        return Rules.PROCEED;
    }

    private static byte[] toByteArray( final File file ) throws IOException
    {
        if( file.length() > Integer.MAX_VALUE )
        {
            throw new IOException( "File is too large to read into memory" );
        }

        byte[] b = new byte[(int) file.length()];
        InputStream in = new FileInputStream( file );
        boolean threw = true;
        try
        {
            read( in, b, 0, b.length );
            threw = false;
        }
        finally
        {
            close( in, threw );
        }

        return b;
    }

    private static int read( InputStream in, byte[] b, int off, int len ) throws IOException
    {
        if( len < 0 )
        {
            throw new IndexOutOfBoundsException( "len is negative" );
        }
        int total = 0;
        while( total < len )
        {
            int result = in.read( b, off + total, len - total );
            if( result == -1 )
            {
                break;
            }
            total += result;
        }
        return total;
    }

    private static void close( InputStream closeable,
                               boolean swallowIOException ) throws IOException
    {
        if( closeable == null )
        {
            return;
        }
        try
        {
            closeable.close();
        }
        catch( IOException e )
        {
            if( swallowIOException )
            {
                Log.getLogger().warn( "IOException thrown while closing Closeable.  " + e.getMessage() );
            }
            else
            {
                throw e;
            }
        }
    }
}