package com.itko.lisa.remote.vse.ext;

import com.itko.lisa.remote.instrument.Rules;
import com.itko.lisa.remote.utils.Log;
import com.itko.lisa.remote.vse.VSEFrame;

import java.io.InputStream;
import java.lang.reflect.Method;

public class ICSSendSessionHelperInterceptor
{

    /**
     * When recording, read the contents of the InputStream and stick it into a
     * string on the frame's arguments before sending it to VSE
     */
    public static boolean onPostRecord( VSEFrame frame,
                                        String method,
                                        String signature,
                                        Object[] args,
                                        GXSDispatchingVSEInterceptor.ParentCaller superCall )
    {
        Log.getLogger().debug( "In onPostRecord of ICSSendSessionHelperInterceptor" );
        if( "doTrustedSendSession".equals( method ) &&
            args[5] instanceof InputStream )
        {
            Log.getLogger().debug( "Handling " + method );
            overrideArguments( frame, args );
        }
        else if( Log.getLogger().isDebugEnabled() )
        {
            Log.getLogger().debug( "Not an expected method.  Saw " + method + " with signature:" + signature + "." );
        }

        superCall.setCallParent( true );
        
        return false;
    }

    /**
     * When playing back, read the contents of the InputStream and stick it into
     * as string on the frame's arguments so that VSE can match correctly
     */
    public static Object onPreHijack( VSEFrame frame,
                                      String method,
                                      String signature,
                                      Object[] args,
                                      GXSDispatchingVSEInterceptor.ParentCaller superCall )
    {
        Log.getLogger().debug( "In onPreHijack of ICSSendSessionHelperInterceptor" );
        if( "doTrustedSendSession".equals( method ) &&
            args[5] instanceof InputStream )
        {
            Log.getLogger().debug( "Handling " + method );
            overrideArguments( frame, args );
        }
        else if( Log.getLogger().isDebugEnabled() )
        {
            Log.getLogger().debug( "Not an expected method.  Saw " + method + " with signature:" + signature + "." );
        }

        superCall.setCallParent( true );

        return Rules.PROCEED;
    }

    /** Reset the last arg from an InputStream to its stringified content */
    private static void overrideArguments( VSEFrame frame, Object[] args )
    {
        try
        {
            Log.getLogger().debug( "Overriding arguments for call" );
            InputStream inputStream = (InputStream) args[5];
            String content = new String();
            int nBytesRead = 0;
            byte[] barrData = new byte[10000];
            if( inputStream.markSupported() )
            {
                inputStream.reset();
            }
            Log.getLogger().debug( "Constructing InputStream content as String" );
            while( (nBytesRead = inputStream.read( barrData, 0, barrData.length )) != -1 )
            {
                content = content + new String( barrData, 0, nBytesRead );
            }

            args[5] = content;
            // Unfortunately setArguments is not public so let's use
            // reflection here
            Method setArgs = VSEFrame.class.getDeclaredMethod( "setArguments",
                                                               new Class[] {Object[].class} );
            setArgs.setAccessible( true );
            setArgs.invoke( frame, new Object[] {args} );
            Log.getLogger().debug( "New arguments set" );
        }
        catch( Exception e )
        {
            Log.getLogger().warn( "Oops, can't transform stream: " + e );
        }
    }
}