package com.itko.lisa.ext.gxs.filter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.itko.lisa.test.FilterBaseImpl;
import com.itko.lisa.test.TestDefException;
import com.itko.lisa.test.TestExec;
import com.itko.lisa.test.TestRunException;
import com.itko.util.Parameter;
import com.itko.util.ParameterList;
import com.itko.util.XMLUtils;
import com.itko.lisa.vse.stateful.model.*;

public class ExtractEventXmlFilter extends FilterBaseImpl {

	private static final long serialVersionUID = -3506334838182674686L;

	private static final transient Log log = LogFactory
			.getLog(ExtractEventXmlFilter.class);

	public static final String FILTER_TITLE = "GXS Extract Event XML Filter";
	private static final String REQUEST_PARAM = "request";
	private static final String REQUEST_LABEL = "VSE Request Property:";
	private String requestProp;
	private static final String PROP_PARAM = "prop";
	private static final String PROP_LABEL = "XML Property Name:";
	private String prop;

	/**
	 * Get the display name for this filter.
	 * 
	 * @return a <code>String</code>
	 */
	public String getTypeName() {
		return FILTER_TITLE;
	}

	/**
	 * Build the list of parameters for this filter.
	 * 
	 * @return a <code>ParameterList</code>
	 */
	public ParameterList getParameters() {
		ParameterList pl = new ParameterList();

		pl.addParameter(new Parameter(REQUEST_LABEL,
									 REQUEST_PARAM, 
									 requestProp,
									 TestExec.PROPERTY_PARAM_TYPE));
		
		pl.addParameter(new Parameter(PROP_LABEL,
									 PROP_PARAM, 
									 prop,
									 String.class));

		return pl;
	}
	
	/**
     * Once constructed, we take the XML Element and get our field data from
     * there.
     * 
     * @param e
     * @throws TestDefException
     */
    public void initialize(org.w3c.dom.Element e) throws TestDefException
    {
        requestProp = XMLUtils.getAttributeOrChildText(e, REQUEST_PARAM);
        prop = XMLUtils.getAttributeOrChildText(e, PROP_PARAM);
    }
    
	@SuppressWarnings("unchecked")
	public Collection getFilterNodeConnections(ParameterList parameterList) {
		return null;
	}
	
    /**
     * @param testExec
     * @return a <code>boolean</code>
     * @throws TestRunException
     */
	@Override
    public boolean subPreFilter(TestExec testExec) throws TestRunException
    {
        // nothing to do here
        return false;
    }
	
	@SuppressWarnings("unchecked")
	@Override
	public boolean subPostFilter(TestExec ts) throws TestRunException {
		List l = (List) ts.getStateValue("lisa.vse.response");
		String body = "";
		if (l instanceof LinkedList) {
			LinkedList ll = (LinkedList)l;
			TransientResponse response = (TransientResponse)ll.getFirst();
		    body = response.getBody();
		} else if (l instanceof ArrayList) {
			ArrayList al = (ArrayList)l;
			TransientResponse response = (TransientResponse)al.get(0);
		    body = response.getBody();
		}

		body = ts.parseInState(body);
		ts.setStateValue(prop, body);
		return false;
	}
}
